@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2007 Sun Microsystems Inc.
+# Copyright 2005-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,15 +14,15 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py 3783 2008-03-28 23:49:14Z wwalker $"
-__version__   = "$Revision: 3783 $"
-__date__      = "$Date: 2008-03-28 19:49:14 -0400 (Fri, 28 Mar 2008) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: gnomespeechfactory.py 3882 2008-05-07 18:22:10Z richb $"
+__version__   = "$Revision: 3882 $"
+__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
@@ -453,12 +453,16 @@ class SpeechServer(speechserver.SpeechServer):
 
         speaker = _Speaker(s._narrow(GNOME.Speech.Speaker))
 
-        # Turn off punctuation if the speaker allows us to do so.  We
-        # do this because we want to handle spoken punctuation on our
-        # own.
+        # Turn off punctuation.  We do this because we want to handle
+        # spoken punctuation on our own.  Only do so if "punctuation mode" 
+        # is available (see # bug #528797).
         #
         try:
-            speaker.setParameterValue("punctuation mode", 0.0)
+            parameters = speaker.getSupportedParameters()
+            for parameter in parameters:
+                if parameter.name == "punctuation mode":
+                    speaker.setParameterValue("punctuation mode", 0.0)
+                    break
         except:
             pass
 
