@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2006-2007 Sun Microsystems Inc.
+# Copyright 2006-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,16 +14,16 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Exposes a dictionary, keynames, that maps key events
 into localized words."""
 
-__id__        = "$Id: keynames.py 3741 2008-03-20 22:07:04Z wwalker $"
-__version__   = "$Revision: 3741 $"
-__date__      = "$Date: 2008-03-20 18:07:04 -0400 (Thu, 20 Mar 2008) $"
-__copyright__ = "Copyright (c) 2006-2007 Sun Microsystems Inc."
+__id__        = "$Id: keynames.py 3882 2008-05-07 18:22:10Z richb $"
+__version__   = "$Revision: 3882 $"
+__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__copyright__ = "Copyright (c) 2006-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import chnames
