@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2007 Sun Microsystems Inc.
+# Copyright 2005-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,15 +14,15 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 3870 2008-04-30 19:33:50Z joanied $"
-__version__   = "$Revision: 3870 $"
-__date__      = "$Date: 2008-04-30 15:33:50 -0400 (Wed, 30 Apr 2008) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id: where_am_I.py 3882 2008-05-07 18:22:10Z richb $"
+__version__   = "$Revision: 3882 $"
+__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import pyatspi
@@ -142,6 +142,15 @@ class WhereAmI:
         elif role == pyatspi.ROLE_TOGGLE_BUTTON:
             self._speakToggleButton(obj, basicOnly)
 
+        elif role == pyatspi.ROLE_SPLIT_PANE:
+            self._speakSplitPane(obj, basicOnly)
+
+        elif role == pyatspi.ROLE_LABEL:
+            self._speakLabel(obj, basicOnly)
+
+        elif role  == pyatspi.ROLE_LAYERED_PANE:
+            self._speakLayeredPane(obj, basicOnly)
+
         else:
             self._speakGenericObject(obj, basicOnly)
 
@@ -259,7 +268,8 @@ class WhereAmI:
         1. label
         2. role
         3. current value
-        4. accelerator (i.e. Alt plus the underlined letter), if any
+        4. selected (if True).
+        5. accelerator (i.e. Alt plus the underlined letter), if any
         """
 
         utterances = []
@@ -272,6 +282,8 @@ class WhereAmI:
         name = self._getObjName(obj)
         utterances.append(name)
         
+        utterances.extend(self._getSpeechForAllTextSelection(obj))
+
         text = self._getObjAccelerator(obj)
         utterances.append(text)
 
@@ -451,6 +463,12 @@ class WhereAmI:
         Gaim, gedit, OpenOffice Writer and Terminal
         """
 
+        cell = self._script.getAncestor(obj,
+                                        [pyatspi.ROLE_TABLE_CELL],
+                                        [pyatspi.ROLE_FRAME])
+        if cell:
+            return self._speakTableCell(cell, basicOnly)
+
         utterances = []
         text = self._getObjLabel(obj)
         utterances.append(text)
@@ -503,9 +521,9 @@ class WhereAmI:
         3. row header of object
         4. object's role
         5. object's contents, if there are multiple columns
-        6. current row (regardless of speak cell/row setting), if
+        6. relative position
+        7. current row (regardless of speak cell/row setting), if
            performing a detailed whereAmI.
-        7. relative position
         8. if expandable/collapsible: expanded/collapsed
         9. if applicable, the level
 
@@ -527,7 +545,6 @@ class WhereAmI:
             nColumns = 0
         else:
             nColumns = table.nColumns
-
             column = table.getColumnAtIndex(obj.getIndexInParent())
             header = table.getColumnHeader(column)
             if header:
@@ -543,7 +560,7 @@ class WhereAmI:
         text = rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
 
-        if nColumns > 1:
+        if nColumns:
             text = self._getTableCell(obj)
             utterances.append(text)
 
@@ -552,43 +569,28 @@ class WhereAmI:
         speech.speakUtterances(utterances)
 
         utterances = []
-        if not basicOnly and table:
-            row = table.getRowAtIndex(
-              orca_state.locusOfFocus.getIndexInParent())
-            # Translators: this in reference to a row in a table.
+        if table:
+            # Translators: this is in references to a column in a
+            # table.
+            text = _("column %d of %d") % ((column + 1), table.nColumns)
+            utterances.append(text)
+
+            # Translators: this is in reference to a row in a table.
             #
-            text = _("row %d of %d") % ((row+1), table.nRows)
+            text = _("row %d of %d") % ((row + 1), table.nRows)
             utterances.append(text)
             speech.speakUtterances(utterances)
 
-        # Speak the current row if performing a "detailed" whereAmI.
-        #
-        if not basicOnly:
-            utterances = self._getTableRow(obj)
-            debug.println(self._debugLevel, \
-                          "second table cell utterances=%s" % \
-                          utterances)
-            speech.speakUtterances(utterances)
-
-        # Speak the remaining items.
+            # Speak the current row if performing a "detailed" whereAmI.
+            #
+            if not basicOnly:
+                utterances = self._getTableRow(obj)
+                debug.println(self._debugLevel, \
+                              "second table cell utterances=%s" % \
+                              utterances)
+                speech.speakUtterances(utterances)
+        
         utterances = []
-
-        if basicOnly:
-            try:
-                table = parent.queryTable()
-            except NotImplementedError:
-                debug.println(self._debugLevel, 
-                              "??? parent=%s" % parent.getRoleName())
-                return
-            else:
-                row = \
-                    table.getRowAtIndex(
-                       orca_state.locusOfFocus.getIndexInParent())
-                # Translators: this in reference to a row in a table.
-                #
-                text = _("row %d of %d") % ((row+1), table.nRows)
-                utterances.append(text)
-
         state = obj.getState()
         if state.contains(pyatspi.STATE_EXPANDABLE):
             if state.contains(pyatspi.STATE_EXPANDED):
@@ -697,38 +699,7 @@ class WhereAmI:
 
         utterances.append(_("Icon panel"))
         utterances.append(self.getObjLabelAndName(obj))
-
-        selectedItems = []
-        totalSelectedItems = 0
-        currentItem = 0
-        for child in panel:
-            state = child.getState()
-            if state.contains(pyatspi.STATE_SELECTED):
-                totalSelectedItems += 1
-                selectedItems.append(child)
-            if state.contains(pyatspi.STATE_FOCUSED):
-                currentItem = child.getIndexInParent() + 1
-
-        # Translators: this is a count of the number of selected icons 
-        # and the count of the total number of icons within an icon panel. 
-        # An example of an icon panel is the Nautilus folder view.
-        #
-        itemString = ngettext("%d of %d item selected",
-                              "%d of %d items selected",
-                              childCount) % \
-                              (totalSelectedItems, childCount)
-        utterances.append(itemString)
-
-        # Translators: this is a indication of the focused icon and the
-        # count of the total number of icons within an icon panel. An
-        # example of an icon panel is the Nautilus folder view.
-        #
-        itemString = _("on item %d of %d") % (currentItem, childCount)
-        utterances.append(itemString)
-
-        if not basicOnly:
-            for i in range(0, len(selectedItems)):
-                utterances.append(self.getObjLabelAndName(selectedItems[i]))
+        utterances.extend(self._getSelectedItemCount(panel, basicOnly))
 
         speech.speakUtterances(utterances)
         
@@ -825,6 +796,133 @@ class WhereAmI:
 
         speech.speakUtterances(utterances)
       
+    def _speakSplitPane(self, obj, basicOnly):
+        """Speak split pane information:
+           1. Name/Label
+           2. Role
+           3. Value
+        """
+
+        utterances = []
+        text = self.getObjLabelAndName(obj)
+        utterances.append(text)
+
+        text = rolenames.getSpeechForRoleName(obj)
+        utterances.append(text)
+
+        valueString = self._script.getTextForValue(obj)
+        utterances.append(valueString)
+
+        speech.speakUtterances(utterances)
+
+    def _speakLabel(self, obj, basicOnly):
+        """Speak label information:
+           1. Name/Label
+           2. selected (if True).
+           3. Role
+        """
+
+        utterances = []
+        text = self.getObjLabelAndName(obj)
+        utterances.append(text)
+
+        utterances.extend(self._getSpeechForAllTextSelection(obj))
+
+        text = rolenames.getSpeechForRoleName(obj)
+        utterances.append(text)
+        speech.speakUtterances(utterances)
+
+    def _speakLayeredPane(self, obj, basicOnly):
+        """Speak layered pane information:
+           1. Name/Label
+           2. Role
+           3. Number of selected items and total number of items.
+        """
+
+        utterances = []
+        text = self.getObjLabelAndName(obj)
+        utterances.append(text)
+
+        text = rolenames.getSpeechForRoleName(obj)
+        utterances.append(text)
+        utterances.extend(self._getSelectedItemCount(obj, basicOnly))
+
+        speech.speakUtterances(utterances)
+
+    def _getSpeechForAllTextSelection(self, obj):
+        """Check if this object has text associated with it and it's
+        completely selected.
+
+        Arguments:
+        - obj: the object being presented
+        """
+
+        utterance = []
+        try:
+            textObj = obj.queryText()
+        except:
+            pass
+        else:
+            noOfSelections = textObj.getNSelections()
+            if noOfSelections == 1:
+                [string, startOffset, endOffset] = \
+                   textObj.getTextAtOffset(0, pyatspi.TEXT_BOUNDARY_LINE_START)
+                if startOffset == 0 and endOffset == len(string):
+                    # Translators: when the user selects (highlights) text in
+                    # a document, Orca lets them know this.
+                    #
+                    # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                    #
+                    utterance = [Q_("text|selected")]
+
+        return utterance
+
+    def _getSelectedItemCount(self, obj, basicOnly):
+        """Return an utterance indicating how many items are selected in this
+        object, the current item, and (if a detailed whereAmI), the names of
+        all the selected items. This object will be an icon panel or a 
+        layered pane.
+
+        Arguments:
+        - obj: the object being presented
+        """
+
+        childCount = obj.childCount
+        selectedItems = []
+        totalSelectedItems = 0
+        currentItem = 0
+        for child in obj:
+            state = child.getState()
+            if state.contains(pyatspi.STATE_SELECTED):
+                totalSelectedItems += 1
+                selectedItems.append(child)
+            if state.contains(pyatspi.STATE_FOCUSED):
+                currentItem = child.getIndexInParent() + 1
+
+        utterances = []
+        # Translators: this is a count of the number of selected icons
+        # and the count of the total number of icons within an icon panel.
+        # An example of an icon panel is the Nautilus folder view.
+        #
+        countString = ngettext("%d of %d item selected",
+                              "%d of %d items selected",
+                              childCount) % \
+                              (totalSelectedItems, childCount)
+        utterances.append(countString)
+
+        # Translators: this is a indication of the focused icon and the
+        # count of the total number of icons within an icon panel. An
+        # example of an icon panel is the Nautilus folder view.
+        #
+        itemString = _("on item %d of %d") % (currentItem, childCount)
+        utterances.append(itemString)
+
+        if not basicOnly:
+            for i in range(0, len(selectedItems)):
+                utterances.append(self.getObjLabelAndName(selectedItems[i]))
+
+        return utterances
+
     def __extractSize(self, uri):
         """Get the http header for a given uri and try to extract the size
         (Content-length).
@@ -1112,7 +1210,11 @@ class WhereAmI:
             pass
         else:
             for i in range(0, action.nActions):
-                if action.getName(i) == "toggle":
+                # Translators: this is the action name for
+                # the 'toggle' action. It must be the same
+                # string used in the *.po file for gail.
+                #
+                if action.getName(i) in ["toggle", _("toggle")]:
                     isToggle = True
                     break
 
@@ -1139,7 +1241,11 @@ class WhereAmI:
                 pass
             else:
                 for i in range(0, action.nActions):
-                    if action.getName(i) == "toggle":
+                    # Translators: this is the action name for
+                    # the 'toggle' action. It must be the same
+                    # string used in the *.po file for gail.
+                    #
+                    if action.getName(i) in ["toggle", _("toggle")]:
                         isToggle = True
                         break
         if not isToggle:
@@ -1402,7 +1508,7 @@ class WhereAmI:
         textOffset = startOffset
 
         for i in range(0, len(line)):
-            attribs = self._getAttributesForChar(text, textOffset, line, i)
+            attribs = self._getAttributesForChar(obj, text, textOffset, line, i)
             debug.println(self._debugLevel,
                           "line attribs <%s>" % (attribs))
             if attribs:
@@ -1417,7 +1523,7 @@ class WhereAmI:
 
         return newLine
 
-    def _getAttributesForChar(self, text, textOffset, line, lineIndex):
+    def _getAttributesForChar(self, obj, text, textOffset, line, lineIndex):
 
         keys = [ "style", "weight", "underline" ]
         attribStr = ""
@@ -1454,6 +1560,15 @@ class WhereAmI:
                             attribStr += " "
                             attribStr += (key + " " + attribute)
 
+            # Also check to see if this is a hypertext link.
+            #
+            if self._script.getLinkIndex(obj, textOffset) >= 0:
+                attribStr += " "
+                # Translators: this indicates that this piece of
+                # text is a hypertext link.
+                #
+                attribStr += _("link")
+
             debug.println(self._debugLevel,
                           "char <%s>: %s" % (line[lineIndex], attribStr))
 
